from twisted.trial.unittest import TestCase
from twisted.python.failure import Failure
from tada import Tadah

class SQLiteTest(TestCase):
    def testConnectFail(self):
        return Tadah("pysqlite2.dbapi2","/tmp/d/i/e/n/o/w/").connect().addBoth(lambda _: True)

    def testConnectPass(self):
        def cursorContext(cursor):
            return True
        return Tadah("pysqlite2.dbapi2",self.mktemp()).connect().addCallback(cursorContext)

    def testGetCursorManually(self):
        def gotCursor(cursor):
            from pysqlite2.dbapi2 import Cursor
            self.assertEquals(cursor.__tada_cursor__.__class__,Cursor)
            return True
        def gotConnection(connection):
            return connection.cursor().addCallback(gotCursor)
        return Tadah("pysqlite2.dbapi2",self.mktemp()).connect().addCallback(gotConnection)

    def testGetCursor(self):
        def gotCursor(cursor):
            from pysqlite2.dbapi2 import Cursor
            self.assertEquals(cursor.__tada_cursor__.__class__,Cursor)
            return True
        return Tadah("pysqlite2.dbapi2",self.mktemp()).cursor().addCallback(gotCursor)

    def testDropNonExistantTable(self):
        def mustFailDeferredResult(result):
            from pysqlite2.dbapi2 import OperationalError
            self.assertEqual(result.value,OperationalError)
        return Tadah("pysqlite2.dbapi2",self.mktemp()).execute("DROP TABLE thisTableShouldNotExist").addBoth(mustFailDeferredResult)

    # # Not (yet) supported by SQLlite
    # def testDropTableIf(self):
    #     return Tadah("pysqlite2.dbapi2",self.mktemp()).execute("DROP TABLE IF EXISTS thisTableShouldNotExist")

    def testCreateTable(self):
        def cursorContext(cursor):
            return True
        def tableCreated(cursor):
            return cursor.execute("DROP TABLE thisTableCouldExist").addCallback(cursorContext)
        def createTable(mightBeCursor):
            if isinstance(mightBeCursor,Failure): mightBeCursor=Tadah("pysqlite2.dbapi2",self.mktemp())
            return mightBeCursor.execute("CREATE TABLE thisTableCouldExist ( id int )").addCallback(lambda _: tableCreated(mightBeCursor))
        return Tadah("pysqlite2.dbapi2",self.mktemp()).execute("DROP TABLE thisTableCouldExist").addBoth(createTable)

